/* ©  Copyright Steve Simon, 2013 */
#include <u.h>
#include <libc.h>
#include <fcall.h>
#include <thread.h>
#include <9p.h>
#include <pool.h>
#include <json.h>
#include "jsonfs.h"
#include "axosoft.h"

static char *Pkgname = "axosoft";
static char *Defmount = "/n/axosoft";

static void
usage(void)
{
	fprint(2, "usage: %s [-a] [-c secs] [-w dir] [-m dir] [-s name] [url]\n", argv0);
	fprint(2, "  -a       don't authenticate\n");
	fprint(2, "  -c secs  filesystem cache time in secs (default 60)\n");
	fprint(2, "  -w dir   dir where webfs is mounted (default /mnt/web)\n");
	fprint(2, "  -m dir   dir where jira will be mounted (default /n/jira)\n");
	fprint(2, "  -s dir   name posted into /srv (none posted by default)\n");
	exits("usage");
}

void
main(int argc, char *argv[])
{
	long tout;
	Sess sess, *s;
	char *mtpt, *svs;
	Node skyhook, *root, *ch;

	quotefmtinstall();
	fmtinstall('F', fcallfmt);
	fmtinstall('D', dirfmt);

	s = &sess;
	memset(s, 0, sizeof(Sess));

	svs = nil;
	mtpt = nil;
	tout = 60;
	s->webfs = "/mnt/web";
	ARGBEGIN{
	case 'D':
		Debug = EARGF(usage());
		break;
	case 'c':
		tout = atoi(EARGF(usage()));
		break;
	case 'w':
		s->webfs = EARGF(usage());
		break;
	case 'm':
		mtpt = EARGF(usage());
		break;
	case 's':
		svs = EARGF(usage());
		break;
	case 'M':
		mainmem->flags |= POOL_NOREUSE|POOL_PARANOIA|POOL_ANTAGONISM;
		break;
	default:
		usage();
	}ARGEND;

	if((s->url = getenv(Pkgname)) == nil)
		sysfatal("%s url not set\n", Pkgname);

	if(strncmp(s->url, "http://", 7) != 0 &&
	   strncmp(s->url, "https://", 8) != 0)
		sysfatal("%s - not a valid url\n", s->url);

	if(dialweb(s, nil) < 0)
		sysfatal("cannot get session to webfs\n");

	if(dialweb(s, nil) < 0)
		sysfatal("cannot get session to webfs\n");

	if(auth(s, Pkgname) < 0)
		sysfatal("quthentication failed, %r\n");

	memset(&skyhook, 0, sizeof(skyhook));
	root = newdir(&skyhook, nil, nil, nil, "");

	ch = newdir(root, nil, nil, nil, "defects");
	ch->scan = defectscan;
	ch->walk = defectwalk;

	ch = newdir(root, nil, nil, nil, "incidents");
	ch->scan = incidentscan;
	ch->walk = incidentwalk;

	ch = newdir(root, nil, nil, nil, "features");
	ch->scan = featurescan;
	ch->walk = featurewalk;

	ch = newdir(root, nil, nil, nil, "tasks");
	ch->scan = taskscan;
	ch->walk = taskwalk;

	ch = newdir(root, nil, nil, nil, "users");
	ch->scan = alluserscan;

	ch = newdir(root, nil, nil, nil, "local");
	ch->scan = localscan;

	ch = newdir(root, nil, nil, nil, "projects");
	ch->scan = projectscan;

	if(!mtpt && !svs)
		mtpt = Defmount;

	startfs(s, &skyhook, tout, Pkgname, svs, mtpt);

	exits(0);
}
